package radioai

import (
	"bytes"
	"crypto/rand"
	"encoding/base64"
	"encoding/json"
	"errors"
	"strings"

	"github.com/coreos/go-etcd/etcd"
)

var (
	masterElectionPath = "/icecast/cluster/master"
	mountPrefix        = "/icecast/mounts/"
	nodePrefix         = "/icecast/nodes/"
)

// A mountpoint for a stream.
type Mount struct {
	// Name (path to the mountpoint).
	Name string

	// Username for source authentication.
	Username string

	// Password for source authentication.
	Password string

	// Fallback stream name (optional).
	Fallback string
}

func mountPath(mountName string) string {
	return mountPrefix + mountName[1:]
}

// RadioAPI is the actual API to the streaming cluster's database.
type RadioAPI struct {
	client *etcd.Client
}

func NewRadioAPI(client *etcd.Client) *RadioAPI {
	return &RadioAPI{client}
}

// GetMount returns data on a specific mountpoint (returns nil if not
// found).
func (r *RadioAPI) GetMount(mountName string) (*Mount, error) {
	response, err := r.client.Get(mountPath(mountName))
	if err != nil {
		return nil, err
	}
	if len(response) != 1 {
		return nil, nil
	}

	var m Mount
	if err := json.NewDecoder(strings.NewReader(response[0].Value)).Decode(&m); err != nil {
		return nil, err
	}
	return &m, nil
}

// SetMount creates or updates a mountpoint.
func (r *RadioAPI) SetMount(m *Mount) error {
	var buf bytes.Buffer
	if err := json.NewEncoder(&buf).Encode(m); err != nil {
		return err
	}

	_, err := r.client.Set(mountPath(m.Name), buf.String(), 0)
	return err
}

// DelMount removes a mountpoint.
func (r *RadioAPI) DelMount(mountName string) error {
	_, err := r.client.Delete(mountPath(mountName))
	return err
}

// ListMounts returns a list of all the configured mountpoints.
func (r *RadioAPI) ListMounts() ([]*Mount, error) {
	response, err := r.client.Get(mountPrefix)
	if err != nil {
		return nil, err
	}
	result := make([]*Mount, 0, len(response))
	for _, entry := range response {
		var m Mount
		if err := json.NewDecoder(strings.NewReader(entry.Value)).Decode(&m); err != nil {
			continue
		}
		result = append(result, &m)
	}
	return result, nil
}

// GetNodes returns the list of active cluster nodes.
func (r *RadioAPI) GetNodes() ([]string, error) {
	response, err := r.client.Get(nodePrefix)
	if err != nil {
		return nil, err
	}
	result := make([]string, 0, len(response))
	for _, entry := range response {
		result = append(result, entry.Value)
	}
	return result, nil
}

// GetMasterAddr returns the address of the current master server.
func (r *RadioAPI) GetMasterAddr() (string, error) {
	response, err := r.client.Get(masterElectionPath)
	if err != nil {
		return "", err
	}
	if len(response) < 1 {
		return "", errors.New("no active master")
	}
	return response[0].Value, nil
}

// GeneratePassword returns a new random password.
func GeneratePassword() string {
	b := make([]byte, 6)
	rand.Read(b)
	return base64.StdEncoding.EncodeToString(b)
}
